#!env python
#
# created: "03:44 22/03-2005" by Kim Petersen <kim@vindinggaard.dk>
#
# $Id$
#
from cgi import escape
from StringIO import StringIO

import PIL.Image

#import Globals
import App.class_init
import OFS.Image

class Thumbnail(OFS.Image.Image):
	"""A thumbnail image - special case with the tag function"""
	meta_class='Thumbnail'

	def tag(self, height=None, width=None, alt=None,
			  scale=0, xscale=0, yscale=0, css_class=None, title=None, **args):
		"""
		Generate an HTML IMG tag for this image, with customization.
		Arguments to self.tag() can be any valid attributes of an IMG tag.
		'src' will always be an absolute pathname, to prevent redundant
		downloading of images. Defaults are applied intelligently for
		'height', 'width', and 'alt'. If specified, the 'scale', 'xscale',
		and 'yscale' keyword arguments will be used to automatically adjust
		the output height and width values of the image tag.
		
		Since 'class' is a Python reserved word, it cannot be passed in
		directly in keyword arguments which is a problem if you are
		trying to use 'tag()' to include a CSS class. The tag() method
		will accept a 'css_class' argument that will be converted to
		'class' in the output tag to work around this.

		(changes here: alt and title are acquired if needed - Kim)
		"""
		if height is None: height=self.height
		if width is None:  width=self.width
		
		# Auto-scaling support
		xdelta = xscale or scale
		ydelta = yscale or scale
		
		if xdelta and width:
			width =  str(int(round(int(width) * xdelta)))
		if ydelta and height:
			height = str(int(round(int(height) * ydelta)))

		result='<img src="%s"' % (self.absolute_url())

		if alt is None:
			alt=getattr(self.aq_parent,'alt','')
		result = '%s alt="%s"' % (result, escape(alt, 1))

		if title is None:
			title=getattr(self.aq_parent,'title','')
		result = '%s title="%s"' % (result, escape(title, 1))
		
		if height:
			result = '%s height="%s"' % (result, height)

		if width:
			result = '%s width="%s"' % (result, width)

         # Collector #1557
			#        if not 'border' in [ x.lower() for x in  args.keys()]:
			#            result = '%s border="0"' % result

		if css_class is not None:
			result = '%s class="%s"' % (result, css_class)

		for key in args.keys():
			value = args.get(key)
			if value:
				result = '%s %s="%s"' % (result, key, value)

		return '%s />' % result

#Globals.default__class_init__(Thumbnail)
App.class_init.default__class_init__(Thumbnail)

class ThumbedImage(OFS.Image.Image):
	"""An Image with thumbnail"""

	meta_type="ThumbedImage"
	
	_properties=OFS.Image.Image._properties+(
		{'id': 'twidth','type': 'int'},
		{'id':'alt', 'type':'string'}, # For some reason this one has been forgotten in OFS.Image
		)

	thumbnail=None
	twidth=128

	def update_data(self,data,content_type=None,size=None):
		OFS.Image.Image.update_data(self,data,content_type,size)
		if data!='': # this is needed when image is created the first time ... to optimize Image class sets it to '' and then calls upload
			self.gen_thumbnail()

	def gen_thumbnail(self):
		"""Generate the thumbnail"""
		ifd=StringIO(str(self.data))
		image=PIL.Image.open(ifd,'r')
		image=image.convert('RGB')
		try:
			theight=int(self.height*(self.twidth/(self.width*1.0)))
		except:
			theight=self.twidth
		image.thumbnail((self.twidth,theight),PIL.Image.ANTIALIAS)
		ofd=StringIO()
		image.save(ofd,"GIF")
		#self.thumbnail=OFS.Image.Image('thumbnail','Thumbnail','').__of__(self) # needed here?
		self.thumbnail=Thumbnail('thumbnail','Thumbnail','').__of__(self) # needed here?
		self.thumbnail.manage_upload(ofd)

		self._p_changed=1

#manage_addThumbedImageForm=Globals.DTMLFile('dtml/imageAdd',globals(),
manage_addThumbedImageForm=App.special_dtml.DTMLFile('dtml/imageAdd',globals(),
														  Kind='ThumbedImage',kind='image')
def manage_addThumbedImage(self, id, file, title='', precondition='', content_type='',
									REQUEST=None):
	"""
	Add a new Thumbed Image object.
	
	Creates a new Thumbed Image object 'id' with the contents of 'file'.

	(nearly 100% copy of OFS/Image's version)
	"""

	id=str(id)
	title=str(title)
	content_type=str(content_type)
	precondition=str(precondition)
	
	id, title = OFS.Image.cookId(id, title, file)
	
	self=self.this()
	
	# First, we create the image without data:
	self._setObject(id, ThumbedImage(id,title,'',content_type, precondition))
	
	# Now we "upload" the data.  By doing this in two steps, we
	# can use a database trick to make the upload more efficient.
	if file:
		self._getOb(id).manage_upload(file)
	if content_type:
		self._getOb(id).content_type=content_type

	if REQUEST is not None:
		try:    url=self.DestinationURL()
		except: url=REQUEST['URL1']
		REQUEST.RESPONSE.redirect('%s/manage_main' % url)
	return id

if __name__=="__main__":
	pass

# Local Variables:
# tab-width: 3
# py-indent-offset: 3
# End:
